@@ -24,20 +24,21 @@
 #include <stdio.h>
 
 #include "ts/ts.h"
-#include "ts/ink_defs.h"
+
+#define PLUGIN_NAME "hello"
 
 void
-TSPluginInit(int argc ATS_UNUSED, const char *argv[] ATS_UNUSED)
+TSPluginInit(int argc, const char *argv[])
 {
   TSPluginRegistrationInfo info;
 
-  info.plugin_name   = "hello-world";
-  info.vendor_name   = "MyCompany";
-  info.support_email = "ts-api-support@MyCompany.com";
+  info.plugin_name   = PLUGIN_NAME;
+  info.vendor_name   = "Apache Software Foundation";
+  info.support_email = "dev@trafficserver.apache.org";
 
   if (TSPluginRegister(&info) != TS_SUCCESS) {
-    TSError("[hello-world] Plugin registration failed.");
+    TSError("[%s] Plugin registration failed", PLUGIN_NAME);
   }
 
-  TSDebug("debug-hello", "Hello World!");
+  TSDebug(PLUGIN_NAME, "Hello World!");
 }
