@@ -597,9 +597,7 @@ EsiParser::clear()
   _parse_start_pos = -1;
 }
 
-EsiParser::~EsiParser()
-{
-}
+EsiParser::~EsiParser() {}
 
 inline void
 EsiParser::_adjustPointers(DocNodeList::iterator node_iter, DocNodeList::iterator end, const char *ext_data_ptr,
