@@ -176,6 +176,7 @@ enum {
   // cache result stats
   http_cache_hit_fresh_stat,
   http_cache_hit_mem_fresh_stat,
+  http_cache_hit_rww_stat,
   http_cache_hit_reval_stat,
   http_cache_hit_ims_stat,
   http_cache_hit_stale_served_stat,
@@ -733,9 +734,10 @@ struct OverridableHttpConfigParams {
 
   MgmtInt background_fill_active_timeout;
 
-  MgmtInt http_chunking_size;   // Maximum chunk size for chunked output.
-  MgmtInt flow_high_water_mark; ///< Flow control high water mark.
-  MgmtInt flow_low_water_mark;  ///< Flow control low water mark.
+  MgmtInt http_chunking_size;          // Maximum chunk size for chunked output.
+  MgmtByte http_drop_chunked_trailers; ///< Whether to drop chunked trailers.
+  MgmtInt flow_high_water_mark;        ///< Flow control high water mark.
+  MgmtInt flow_low_water_mark;         ///< Flow control low water mark.
 
   MgmtInt default_buffer_size_index;
   MgmtInt default_buffer_water_mark;
@@ -847,7 +849,7 @@ public:
   MgmtByte referer_filter_enabled  = 0;
   MgmtByte referer_format_redirect = 0;
 
-  MgmtByte strict_uri_parsing = 0;
+  MgmtByte strict_uri_parsing = 2;
 
   MgmtByte reverse_proxy_enabled = 0;
   MgmtByte url_remap_required    = 1;
@@ -873,6 +875,7 @@ public:
 
   MgmtInt body_factory_response_max_size = 8192;
 
+  MgmtByte scheme_proto_mismatch_policy = 2;
   // noncopyable
   /////////////////////////////////////
   // operator = and copy constructor //
