@@ -166,6 +166,21 @@ static const StaticTable STATIC_TABLE[] = {{"", ""},
                                            {"via", ""},
                                            {"www-authenticate", ""}};
 
+/**
+  Threshold for total HdrHeap size which used by HPAK Dynamic Table.
+  The HdrHeap is filled by MIMEHdrImpl and MIMEFieldBlockImpl like below.
+  This threshold allow to allocate 3 HdrHeap at maximum.
+
+                     +------------------+-----------------------------+
+   HdrHeap 1 (2048): | MIMEHdrImpl(592) | MIMEFieldBlockImpl(528) x 2 |
+                     +------------------+-----------------------------+--...--+
+   HdrHeap 2 (4096): | MIMEFieldBlockImpl(528) x 7                            |
+                     +------------------------------------------------+--...--+--...--+
+   HdrHeap 3 (8192): | MIMEFieldBlockImpl(528) x 15                                   |
+                     +------------------------------------------------+--...--+--...--+
+*/
+static constexpr uint32_t HPACK_HDR_HEAP_THRESHOLD = sizeof(MIMEHdrImpl) + sizeof(MIMEFieldBlockImpl) * (2 + 7 + 15);
+
 /******************
  * Local functions
  ******************/
@@ -312,16 +327,34 @@ HpackIndexingTable::size() const
   return _dynamic_table->size();
 }
 
-bool
+void
 HpackIndexingTable::update_maximum_size(uint32_t new_size)
 {
-  return _dynamic_table->update_maximum_size(new_size);
+  _dynamic_table->update_maximum_size(new_size);
+}
+
+//
+// HpackDynamicTable
+//
+HpackDynamicTable::~HpackDynamicTable()
+{
+  this->_headers.clear();
+
+  this->_mhdr->fields_clear();
+  this->_mhdr->destroy();
+  delete this->_mhdr;
+
+  if (this->_mhdr_old != nullptr) {
+    this->_mhdr_old->fields_clear();
+    this->_mhdr_old->destroy();
+    delete this->_mhdr_old;
+  }
 }
 
 const MIMEField *
 HpackDynamicTable::get_header_field(uint32_t index) const
 {
-  return _headers.at(index);
+  return this->_headers.at(index);
 }
 
 void
@@ -337,28 +370,25 @@ HpackDynamicTable::add_header_field(const MIMEField *field)
     // It is not an error to attempt to add an entry that is larger than
     // the maximum size; an attempt to add an entry larger than the entire
     // table causes the table to be emptied of all existing entries.
-    _headers.clear();
-    _mhdr->fields_clear();
-    _current_size = 0;
-  } else {
-    _current_size += header_size;
-    while (_current_size > _maximum_size) {
-      int last_name_len, last_value_len;
-      MIMEField *last_field = _headers.back();
+    this->_headers.clear();
+    this->_mhdr->fields_clear();
 
-      last_field->name_get(&last_name_len);
-      last_field->value_get(&last_value_len);
-      _current_size -= ADDITIONAL_OCTETS + last_name_len + last_value_len;
-
-      _headers.erase(_headers.begin() + _headers.size() - 1);
-      _mhdr->field_delete(last_field, false);
+    if (this->_mhdr_old) {
+      this->_mhdr_old->fields_clear();
+      this->_mhdr_old->destroy();
+      delete this->_mhdr_old;
+      this->_mhdr_old = nullptr;
     }
 
-    MIMEField *new_field = _mhdr->field_create(name, name_len);
-    new_field->value_set(_mhdr->m_heap, _mhdr->m_mime, value, value_len);
-    _mhdr->field_attach(new_field);
-    // XXX Because entire Vec instance is copied, Its too expensive!
-    _headers.insert(_headers.begin(), new_field);
+    this->_current_size = 0;
+  } else {
+    this->_current_size += header_size;
+    this->_evict_overflowed_entries();
+
+    MIMEField *new_field = this->_mhdr->field_create(name, name_len);
+    new_field->value_set(this->_mhdr->m_heap, this->_mhdr->m_mime, value, value_len);
+    this->_mhdr->field_attach(new_field);
+    this->_headers.push_front(new_field);
   }
 }
 
@@ -381,118 +411,70 @@ HpackDynamicTable::size() const
 // are evicted from the end of the header table until the size of the
 // header table is less than or equal to the maximum size.
 //
-bool
+void
 HpackDynamicTable::update_maximum_size(uint32_t new_size)
 {
-  while (_current_size > new_size) {
-    if (_headers.size() <= 0) {
-      return false;
-    }
-    int last_name_len, last_value_len;
-    MIMEField *last_field = _headers.back();
-
-    last_field->name_get(&last_name_len);
-    last_field->value_get(&last_value_len);
-    _current_size -= ADDITIONAL_OCTETS + last_name_len + last_value_len;
-
-    _headers.erase(_headers.begin() + _headers.size() - 1);
-    _mhdr->field_delete(last_field, false);
-  }
-
-  _maximum_size = new_size;
-  return true;
+  this->_maximum_size = new_size;
+  this->_evict_overflowed_entries();
 }
 
 uint32_t
 HpackDynamicTable::length() const
 {
-  return _headers.size();
+  return this->_headers.size();
 }
 
-//
-// [RFC 7541] 5.1. Integer representation
-//
-int64_t
-encode_integer(uint8_t *buf_start, const uint8_t *buf_end, uint32_t value, uint8_t n)
+void
+HpackDynamicTable::_evict_overflowed_entries()
 {
-  if (buf_start >= buf_end) {
-    return -1;
+  if (this->_current_size <= this->_maximum_size) {
+    // Do nothing
+    return;
   }
 
-  uint8_t *p = buf_start;
+  for (auto h = this->_headers.rbegin(); h != this->_headers.rend(); ++h) {
+    int name_len, value_len;
+    (*h)->name_get(&name_len);
+    (*h)->value_get(&value_len);
 
-  if (value < (static_cast<uint32_t>(1 << n) - 1)) {
-    *(p++) = value;
+    this->_current_size -= ADDITIONAL_OCTETS + name_len + value_len;
+
+    if (this->_mhdr_old && this->_mhdr_old->fields_count() != 0) {
+      this->_mhdr_old->field_delete(*h, false);
+    } else {
+      this->_mhdr->field_delete(*h, false);
+    }
+
+    this->_headers.pop_back();
+
+    if (this->_current_size <= this->_maximum_size) {
+      break;
+    }
+  }
+
+  this->_mime_hdr_gc();
+}
+
+/**
+   When HdrHeap size of current MIMEHdr exceeds the threshold, allocate new MIMEHdr and HdrHeap.
+   The old MIMEHdr and HdrHeap will be freed, when all MIMEFiled are deleted by HPACK Entry Eviction.
+ */
+void
+HpackDynamicTable::_mime_hdr_gc()
+{
+  if (this->_mhdr_old == nullptr) {
+    if (this->_mhdr->m_heap->total_used_size() >= HPACK_HDR_HEAP_THRESHOLD) {
+      this->_mhdr_old = this->_mhdr;
+      this->_mhdr     = new MIMEHdr();
+      this->_mhdr->create();
+    }
   } else {
-    *(p++) = (1 << n) - 1;
-    value -= (1 << n) - 1;
-    while (value >= 128) {
-      if (p >= buf_end) {
-        return -1;
-      }
-      *(p++) = (value & 0x7F) | 0x80;
-      value  = value >> 7;
+    if (this->_mhdr_old->fields_count() == 0) {
+      this->_mhdr_old->destroy();
+      delete this->_mhdr_old;
+      this->_mhdr_old = nullptr;
     }
-    if (p + 1 >= buf_end) {
-      return -1;
-    }
-    *(p++) = value;
   }
-  return p - buf_start;
-}
-
-int64_t
-encode_string(uint8_t *buf_start, const uint8_t *buf_end, const char *value, size_t value_len)
-{
-  uint8_t *p       = buf_start;
-  bool use_huffman = true;
-  char *data       = nullptr;
-  int64_t data_len = 0;
-
-  // TODO Choose whether to use Huffman encoding wisely
-
-  if (use_huffman && value_len) {
-    data = static_cast<char *>(ats_malloc(value_len * 4));
-    if (data == nullptr) {
-      return -1;
-    }
-    data_len = huffman_encode(reinterpret_cast<uint8_t *>(data), reinterpret_cast<const uint8_t *>(value), value_len);
-  }
-
-  // Length
-  const int64_t len = encode_integer(p, buf_end, data_len, 7);
-  if (len == -1) {
-    if (use_huffman) {
-      ats_free(data);
-    }
-
-    return -1;
-  }
-
-  if (use_huffman) {
-    *p |= 0x80;
-  }
-  p += len;
-
-  if (buf_end < p || buf_end - p < data_len) {
-    if (use_huffman) {
-      ats_free(data);
-    }
-
-    return -1;
-  }
-
-  // Value
-  if (data_len) {
-    memcpy(p, data, data_len);
-    p += data_len;
-  }
-
-  if (use_huffman) {
-    ats_free(data);
-  }
-
-  return p - buf_start;
 }
 
 int64_t
@@ -505,7 +487,7 @@ encode_indexed_header_field(uint8_t *buf_start, const uint8_t *buf_end, uint32_t
   uint8_t *p = buf_start;
 
   // Index
-  const int64_t len = encode_integer(p, buf_end, index, 7);
+  const int64_t len = xpack_encode_integer(p, buf_end, index, 7);
   if (len == -1) {
     return -1;
   }
@@ -551,7 +533,8 @@ encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t
   }
 
   // Index
-  len = encode_integer(p, buf_end, index, prefix);
+  *p  = 0;
+  len = xpack_encode_integer(p, buf_end, index, prefix);
   if (len == -1) {
     return -1;
   }
@@ -566,7 +549,7 @@ encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t
   // Value String
   int value_len;
   const char *value = header.value_get(&value_len);
-  len               = encode_string(p, buf_end, value, value_len);
+  len               = xpack_encode_string(p, buf_end, value, value_len);
   if (len == -1) {
     return -1;
   }
@@ -616,7 +599,7 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
   }
 
   // Name String
-  len = encode_string(p, buf_end, lower_name, name_len);
+  len = xpack_encode_string(p, buf_end, lower_name, name_len);
   if (len == -1) {
     return -1;
   }
@@ -625,7 +608,7 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
   // Value String
   int value_len;
   const char *value = header.value_get(&value_len);
-  len               = encode_string(p, buf_end, value, value_len);
+  len               = xpack_encode_string(p, buf_end, value, value_len);
   if (len == -1) {
     return -1;
   }
@@ -639,95 +622,15 @@ encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf
 int64_t
 encode_dynamic_table_size_update(uint8_t *buf_start, const uint8_t *buf_end, uint32_t size)
 {
-  const int64_t len = encode_integer(buf_start, buf_end, size, 5);
+  buf_start[0]      = 0x20;
+  const int64_t len = xpack_encode_integer(buf_start, buf_end, size, 5);
   if (len == -1) {
     return -1;
   }
-  buf_start[0] |= 0x20;
 
   return len;
 }
 
-//
-// [RFC 7541] 5.1. Integer representation
-//
-int64_t
-decode_integer(uint32_t &dst, const uint8_t *buf_start, const uint8_t *buf_end, uint8_t n)
-{
-  if (buf_start >= buf_end) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
-
-  const uint8_t *p = buf_start;
-
-  dst = (*p & ((1 << n) - 1));
-  if (dst == static_cast<uint32_t>(1 << n) - 1) {
-    int m = 0;
-    do {
-      if (++p >= buf_end) {
-        return HPACK_ERROR_COMPRESSION_ERROR;
-      }
-
-      uint32_t added_value = *p & 0x7f;
-      if ((UINT32_MAX >> m) < added_value) {
-        // Excessively large integer encodings - in value or octet
-        // length - MUST be treated as a decoding error.
-        return HPACK_ERROR_COMPRESSION_ERROR;
-      }
-      dst += added_value << m;
-      m += 7;
-    } while (*p & 0x80);
-  }
-
-  return p - buf_start + 1;
-}
-
-//
-// [RFC 7541] 5.2. String Literal Representation
-// return content from String Data (Length octets) with huffman decoding if it is encoded
-//
-int64_t
-decode_string(Arena &arena, char **str, uint32_t &str_length, const uint8_t *buf_start, const uint8_t *buf_end)
-{
-  if (buf_start >= buf_end) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
-
-  const uint8_t *p            = buf_start;
-  bool isHuffman              = *p & 0x80;
-  uint32_t encoded_string_len = 0;
-  int64_t len                 = 0;
-
-  len = decode_integer(encoded_string_len, p, buf_end, 7);
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
-  p += len;
-
-  if ((p + encoded_string_len) > buf_end) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
-
-  if (isHuffman) {
-    // Allocate temporary area twice the size of before decoded data
-    *str = arena.str_alloc(encoded_string_len * 2);
-
-    len = huffman_decode(*str, p, encoded_string_len);
-    if (len < 0) {
-      return HPACK_ERROR_COMPRESSION_ERROR;
-    }
-    str_length = len;
-  } else {
-    *str = arena.str_alloc(encoded_string_len);
-
-    memcpy(*str, reinterpret_cast<const char *>(p), encoded_string_len);
-
-    str_length = encoded_string_len;
-  }
-
-  return p + encoded_string_len - buf_start;
-}
-
 //
 // [RFC 7541] 6.1. Indexed Header Field Representation
 //
@@ -735,11 +638,11 @@ int64_t
 decode_indexed_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start, const uint8_t *buf_end,
                             HpackIndexingTable &indexing_table)
 {
-  uint32_t index = 0;
+  uint64_t index = 0;
   int64_t len    = 0;
 
-  len = decode_integer(index, buf_start, buf_end, 7);
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+  len = xpack_decode_integer(index, buf_start, buf_end, 7);
+  if (len == XPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -771,22 +674,22 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
 {
   const uint8_t *p         = buf_start;
   bool isIncremental       = false;
-  uint32_t index           = 0;
+  uint64_t index           = 0;
   int64_t len              = 0;
   HpackField ftype         = hpack_parse_field_type(*p);
   bool has_http2_violation = false;
 
   if (ftype == HpackField::INDEXED_LITERAL) {
-    len           = decode_integer(index, p, buf_end, 6);
+    len           = xpack_decode_integer(index, p, buf_end, 6);
     isIncremental = true;
   } else if (ftype == HpackField::NEVERINDEX_LITERAL) {
-    len = decode_integer(index, p, buf_end, 4);
+    len = xpack_decode_integer(index, p, buf_end, 4);
   } else {
     ink_assert(ftype == HpackField::NOINDEX_LITERAL);
-    len = decode_integer(index, p, buf_end, 4);
+    len = xpack_decode_integer(index, p, buf_end, 4);
   }
 
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+  if (len == XPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -801,10 +704,10 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
     }
   } else {
     char *name_str        = nullptr;
-    uint32_t name_str_len = 0;
+    uint64_t name_str_len = 0;
 
-    len = decode_string(arena, &name_str, name_str_len, p, buf_end);
-    if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+    len = xpack_decode_string(arena, &name_str, name_str_len, p, buf_end);
+    if (len == XPACK_ERROR_COMPRESSION_ERROR) {
       return HPACK_ERROR_COMPRESSION_ERROR;
     }
 
@@ -823,10 +726,10 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
 
   // Decode header field value
   char *value_str        = nullptr;
-  uint32_t value_str_len = 0;
+  uint64_t value_str_len = 0;
 
-  len = decode_string(arena, &value_str, value_str_len, p, buf_end);
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+  len = xpack_decode_string(arena, &value_str, value_str_len, p, buf_end);
+  if (len == XPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -869,9 +772,9 @@ update_dynamic_table_size(const uint8_t *buf_start, const uint8_t *buf_end, Hpac
   }
 
   // Update header table size if its required.
-  uint32_t size = 0;
-  int64_t len   = decode_integer(size, buf_start, buf_end, 5);
-  if (len == HPACK_ERROR_COMPRESSION_ERROR) {
+  uint64_t size = 0;
+  int64_t len   = xpack_decode_integer(size, buf_start, buf_end, 5);
+  if (len == XPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -879,9 +782,7 @@ update_dynamic_table_size(const uint8_t *buf_start, const uint8_t *buf_end, Hpac
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
-  if (indexing_table.update_maximum_size(size) == false) {
-    return HPACK_ERROR_COMPRESSION_ERROR;
-  }
+  indexing_table.update_maximum_size(size);
 
   return len;
 }
@@ -946,11 +847,7 @@ hpack_decode_header_block(HpackIndexingTable &indexing_table, HTTPHdr *hdr, cons
 
     field->name_get(&name_len);
     field->value_get(&value_len);
-
-    // [RFC 7540] 6.5.2. SETTINGS_MAX_HEADER_LIST_SIZE:
-    // The value is based on the uncompressed size of header fields, including the length of the name and value in octets plus an
-    // overhead of 32 octets for each header field.
-    total_header_size += name_len + value_len + ADDITIONAL_OCTETS;
+    total_header_size += name_len + value_len;
 
     if (total_header_size > max_header_size) {
       return HPACK_ERROR_SIZE_EXCEEDED_ERROR;
